@@ -14,7 +14,7 @@ Daemon::Control->new({
     program     => sub { sleep $_[1] },
     program_args => [ 10 ],
 
-    pid_file    => '/dev/null', # I don't want to leave tmp files for testing.
+    pid_file    => 'pid_tmp',
     stderr_file => '/dev/null',
     stdout_file => '/dev/null',
 
